import {
  Body,
  Controller,
  ForbiddenException,
  Get,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionDiaService } from './institucion-dia.service';
import { Operador } from 'src/operador/entity/operador.entity';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { UpdateInstitucionDiaDto } from './dto/input/update.dto';
import { InstitucionCarreaOutputDto } from './dto/output/institucion-carrera.dto';

@Controller('institucion-dia')
@ApiTags('institucion-dia')
export class InstitucionDiaController {
  constructor(private institucionDiaService: InstitucionDiaService) {}

  @Serealize(InstitucionCarreaOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna todos los días de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  dias(@Query() query: IdInstitucionDto) {
    return this.institucionDiaService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Serealize(InstitucionCarreaOutputDto)
  @Get('hoy')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna la información del día de hoy de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  hoy(@Query() query: IdInstitucionDto) {
    return this.institucionDiaService.hoy(parseInt(query.id_institucion));
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que actualiza la información de un día de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description:
      'Todas las variables a excepción de id_institucion_dia son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_institucion_dia: 217,
          _activo: true,
          _hora_fin: '17:45',
          _hora_inicio: '09:00',
        },
      },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionDiaDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ForbiddenException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.institucionDiaService.update(admin, body);
  }
}
