import {
  Body,
  ConflictException,
  Controller,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionInfraccionService } from './institucion-infraccion.service';
import { Operador } from 'src/operador/entity/operador.entity';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { CreateInstitucionInfraccionDto } from './dto/input/create.dto';
import { UpdateInstitucionInfraccionDto } from './dto/input/update.dto';
import { InfraccionOutputDto } from './dto/output/infraccion.dto';
import { InstitucionInfraccionOutputDto } from './dto/output/institucion-infraccion.dto';

@Controller('institucion-infraccion')
@ApiTags('institucion-infraccion')
export class InstitucionInfraccionController {
  constructor(
    private institucionInfraccionService: InstitucionInfraccionService,
  ) {}

  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que crea una infracción.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Es obligatorio mandar la variable infracción.',
    examples: { ejemplo: { value: { infraccion: '' } } },
  })
  create(@Request() req, @Body() body: CreateInstitucionInfraccionDto) {
    const superAdmin: Operador = req.user.operador;

    if (!superAdmin || superAdmin.tipoUsuario.id_tipo_usuario != 2)
      throw new ConflictException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.institucionInfraccionService.create(body.infraccion);
  }

  @Serealize(InfraccionOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que retorna todas las infracciones.' })
  @ApiBearerAuth('jwt')
  get() {
    return this.institucionInfraccionService.findAll();
  }

  @Serealize(InstitucionInfraccionOutputDto)
  @Get('infracciones')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna las infracciones de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  infracciones(@Query() query: IdInstitucionDto) {
    return this.institucionInfraccionService.findAllByIdInstitucion(
      parseInt(query.id_institucion),
    );
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que actualiza los días de multa de una infracción.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Las dos variables son obligatorias.',
    examples: {
      ejemplo: { value: { id_institucion_infraccion: 603, dias_multa: 7 } },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionInfraccionDto) {
    const admin: Operador = req.user.operador;

    if (!admin || admin.tipoUsuario.id_tipo_usuario != 3)
      throw new ConflictException(
        'No tienes los permisos necesarios para realizar esta acción.',
      );
    return this.institucionInfraccionService.update(admin, body);
  }
}
